"""
Call by python test3Assoc.py testType savePath
If testType = 1, it probes (dog & bites) -> bones
If testType = 2, it probes (cat & plays) -> yarn
If testType = 3, it probes (prime & dog) -> mammals 
If testType = 4, it probes (prime & plays & yarn) -> (cats & mammals)
"""
import sys
import numpy as np
from FSA_BrainScaleS import FSA_BrainScaleS
from pysthal.command_line_util import init_logger
import pylogging

#--ARGS STEP
args = sys.argv
numberArgs = args.__len__()
if (numberArgs == 3):
    testType = int(args[1])
    savePath = args[2]
else:
    print "****Error*** This needs a test name"

#--STARTING STEP
FSABSS = FSA_BrainScaleS()
logger = pylogging.get("guidebook")

init_logger("WARN", [
    ("guidebook", "DEBUG"),
    ("marocco", "DEBUG"),
    #("Calibtic", "DEBUG"),
    #("sthal", "INFO")
])

#--SETUP NETWORK STEP
FSABSS.brainScalesHeader()
simTime = 500.0
pop = FSABSS.allocateNeurons(240) # 64 n/h (512 c/w, 8 c/n)
FSABSS.printParameters(pop)
pop.record()

cAProjections = []
cAProjections = cAProjections + [FSABSS.makeCA(pop,0)]# mammals
cAProjections = cAProjections + [FSABSS.makeCA(pop,1)]# dog
cAProjections = cAProjections + [FSABSS.makeCA(pop,2)]# cat
cAProjections = cAProjections + [FSABSS.makeCA(pop,3)]# have
cAProjections = cAProjections + [FSABSS.makeCA(pop,4)]# bites
cAProjections = cAProjections + [FSABSS.makeCA(pop,5)]# plays
cAProjections = cAProjections + [FSABSS.makeCA(pop,6)]# fur
cAProjections = cAProjections + [FSABSS.makeCA(pop,7)]# bone
cAProjections = cAProjections + [FSABSS.makeCA(pop,8)]# yarn
cAProjections = cAProjections + [FSABSS.makeCA(pop,9)]# PRIME

#--INHERITANCE STEP
# dog and cat are mammals
halfProjections = []
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,9,pop,0)] # prime > base
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,1,pop,0)] # dog > mamml
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,2,pop,0)] # cat > mamml

#--THREE ASSOCIATION STEP
# dog bites bone
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,1,pop,4)] #dog > bites
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,1,pop,7)] #dog > bone
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,4,pop,1)] #bite > dog
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,4,pop,7)] #bite > bone
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,7,pop,1)] #bone > dog
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,7,pop,4)] #bone > bite

# cat plays yarn
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,2,pop,5)] #cat > plays
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,2,pop,8)] #cat > yarn
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,5,pop,2)] #plays > cat 
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,5,pop,8)] #plays > yarn 
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,8,pop,2)] #yarn > cat 
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,8,pop,2)] #yarn > plays

# mammals have fur
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,0,pop,3)] #mammal > have
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,0,pop,6)] #mammal > fur
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,3,pop,0)] #have > mammal
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,3,pop,6)] #have > fur
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,6,pop,0)] #fur > mammal
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,6,pop,3)] #fur > have

#-- TEST STEP
excSpikeTimes0 = [100]
spikeSources0 = FSABSS.getSpikeSources(excSpikeTimes0)

excSpikeTimes1 = [200]
spikeSources1 = FSABSS.getSpikeSources(excSpikeTimes1)
 
excSpikeTimes2 = [300]
spikeSources2 = FSABSS.getSpikeSources(excSpikeTimes2)


sourceProjections = []
if (testType == 1):
    # test 3 associations: (dog & bites) -> bones
    sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources0,pop,1)]
    sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources1,pop,4)]

elif (testType == 2):
    # test 3 associations: (cat & plays) -> yarn
    sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources0,pop,2)]
    sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources1,pop,5)]

elif (testType == 3):
    # test hierarchy: (prime & dog) -> mammals 
    sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources0,pop,9)]
    sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources1,pop,1)]

elif (testType == 4):
    # test hierarchy: (prime & plays & yarn) -> (cats & mammals) 
    sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources0,pop,9)]
    sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources1,pop,5)]
    sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources2,pop,8)]

elif (testType == 5):
    # test hierarchy: (mammal) ->  
    sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources0,pop,0)]


else:
    print "Insert the testType"

#-- HICANNS SELECTION ON WAFER
FSABSS.brainScalesMap(pop,[295,296,297,298]) # 240 n (9 CAs, 1 CA = 20n)
FSABSS.runSim(10) # run pynn after mapping

#-- WEIGHTS SETTINGS ON WAFER
# note that testType selected the specific inputWeight
FSABSS.setHardwareWeights(cAProjections,sourceProjections,halfProjections,[],
                          [],[],[])

#-- FINAL STEP
FSABSS.brainScalesFooter()
FSABSS.runSim(simTime)
np.savetxt(savePath + "_{}".format(testType) + ".txt", pop.getSpikes())
FSABSS.resetSim()



