# This has a three state automata.  When the first and second states are both
# ignited, they turn on the third state, which in turns shuts them down.

# The output should show the first state coming on around 100, and running until
# after 300ms.  The second state comes on around 300ms supporting a transition
# to the third state.  That, and that alone, persists until the end.

#Import the FSA class.  That does the bulk of the work. 
from FSA_BrainScaleS import FSA_BrainScaleS

#import os
#import sys
import numpy as np  #import numpy for the save text

#import the logging commands for printing run time debug information.
from pysthal.command_line_util import init_logger
import pylogging

#---STEP 2 main--
#Create an instance of the fsa class.
FSABSS = FSA_BrainScaleS()

#turn on logging
logger = pylogging.get("guidebook")
init_logger("WARN", [
    ("guidebook", "DEBUG"),
    ("marocco", "DEBUG"),
    #("Calibtic", "DEBUG"),
    #("sthal", "INFO")
])

#Calls for brainscales initialisation
FSABSS.brainScalesHeader()

#This is used to specify how long the simulation will run.
duration = 800.0


#SETUP NETWORK
pop = FSABSS.allocateNeurons(60)
#FSABSS.printParameters(pop)
pop.record()
#pynn.PopulationView(pop, [5]).record_v()
#pynn.PopulationView(pop, [15]).record_v()

#Make synapses that support CA reverberation.  These later need to
#be set on the hardware.
cAProjections = []
cAProjections = cAProjections + [FSABSS.makeCA(pop,0)]
cAProjections = cAProjections + [FSABSS.makeCA(pop,1)]
cAProjections = cAProjections + [FSABSS.makeCA(pop,2)]

#Set up spike source (in this case one spike each) for starting states 0 and 1.
excSpikeTimes0 = [100]
spikeSources0 = FSABSS.getSpikeSources(excSpikeTimes0)
excSpikeTimes1 = [300]
spikeSources1 = FSABSS.getSpikeSources(excSpikeTimes1)

#Connect the spike sources to the states.
sourceProjections = []
sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources0,pop,0)]
sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources1,pop,1)]

#Set up the forward synapses so that 0 and 1 can turn on 2 together, but
#not individually.
halfProjections = []
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,0,pop,2)]
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,1,pop,2)]

#Set up the backward synapses so that 2 can shut down 0 and 1.
stopProjections = []
stopProjections = stopProjections + [FSABSS.stateStopsState(pop,2,pop,0)]
stopProjections = stopProjections + [FSABSS.stateStopsState(pop,2,pop,1)]

#Allocate the population to a particularly hicann
FSABSS.brainScalesMap(pop,[297])

#Run pynn after mapping that sets up the gross topology.
FSABSS.runSim(10) 

#print FSABSS.marocco.stats
#print FSABSS.marocco.stats.getSynapses()
#print FSABSS.marocco.stats.getSynapseLoss()

#Set the hardware weights.
FSABSS.setHardwareWeights(cAProjections,sourceProjections,halfProjections,[],
                          stopProjections,[],[])


#Calls for brainscales initialisation after the synaptic weights have actually
# been set.
FSABSS.brainScalesFooter()

#Run properly.
FSABSS.runSim(duration)

#Printout the results.
np.savetxt("output/test3StateSpikes.txt", pop.getSpikes())
FSABSS.resetSim()
