# Type 2 Network
#
# Legenda: Sn=sourceN, CAn=CellAsseblyN, + means ignition, - means hinibition, 1/2 means half connections 
# 
# S0+CA0-1/2-CA2
# CA1-1/2-CA2
# CA2-CA0
# CA2-CA1
#
# The output should show the first state coming on around 100, and running until
# after 300ms.  The second state comes on around 300ms supporting a transition
# to the third state.  That, and that alone, persists until the end around 
# 600ms.  This should happen 10 times (it's a parameter in the py file).

#---STEP 1
from FSA_BrainScaleS import FSA_BrainScaleS

import os
import sys
import numpy as np

from pysthal.command_line_util import init_logger
import pylogging

#---STEP 2 main--
timesToRun = 10
epochLength = 1000
FSABSS = FSA_BrainScaleS()

logger = pylogging.get("guidebook")

init_logger("WARN", [
    ("guidebook", "DEBUG"),
    ("marocco", "DEBUG"),
    #("Calibtic", "DEBUG"),
    #("sthal", "INFO")
])

FSABSS.brainScalesHeader()
duration = epochLength*timesToRun

#SETUP NETWORK
pop = FSABSS.allocateNeurons(60)
FSABSS.printParameters(pop)
pop.record()

cAProjections = []
cAProjections = cAProjections + [FSABSS.makeCA(pop,0)]
cAProjections = cAProjections + [FSABSS.makeCA(pop,1)]
cAProjections = cAProjections + [FSABSS.makeCA(pop,2)]

excSpikeTimes0 = []
excSpikeTimes1 = []
inhSpikeTimes0 = []
for epoch in range (0,timesToRun):
    excSpikeTimes0 = excSpikeTimes0 + [(epochLength*epoch)+100]
    excSpikeTimes1 = excSpikeTimes1 + [(epochLength*epoch)+300]
    inhSpikeTimes0 = inhSpikeTimes0 + [(epochLength*epoch)+600, (epochLength*epoch)+610]

spikeSources0 = FSABSS.getSpikeSources(excSpikeTimes0)
spikeSources1 = FSABSS.getSpikeSources(excSpikeTimes1)
spikeSources2 = FSABSS.getSpikeSources(inhSpikeTimes0)

sourceProjections = []
sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources0,pop,0)]
sourceProjections = sourceProjections + [FSABSS.turnOnStateFromSpikeSource(spikeSources1,pop,1)]

halfProjections = []
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,0,pop,2)]
halfProjections = halfProjections + [FSABSS.stateHalfStartsState(pop,1,pop,2)]

stopProjections = []
stopProjections = stopProjections + [FSABSS.stateStopsState(pop,2,pop,0)]
stopProjections = stopProjections + [FSABSS.stateStopsState(pop,2,pop,1)]

sourceStopProjections = []
sourceStopProjections = sourceStopProjections+ [FSABSS.turnOffStateFromSpikeSource(spikeSources2,pop,0)]
sourceStopProjections = sourceStopProjections+ [FSABSS.turnOffStateFromSpikeSource(spikeSources2,pop,1)]
sourceStopProjections = sourceStopProjections+ [FSABSS.turnOffStateFromSpikeSource(spikeSources2,pop,2)]

FSABSS.brainScalesMap(pop,[297])
FSABSS.runSim(10) # run pynn after mapping

#Set the hardware weights and run properly.
FSABSS.setHardwareWeights(cAProjections,sourceProjections,halfProjections,[],
                          stopProjections,sourceStopProjections,[])

FSABSS.brainScalesFooter()
FSABSS.runSim(duration)
np.savetxt("output/test3MultSpikes.txt", pop.getSpikes())
FSABSS.resetSim()

