# Turn left, turn right, or move forward
# The first spike source starts the parse, the second presents turn or move.
# The third presents left right or forward
# Change the comment for that (look for select and pick a sent).
#
# The output are in ouput/move*.txt 
# They should be the start state from 100 until around 300.  The
# verb is then switched on (turn or move in word1), switching to state
# 1 or 2.  The adj is then switched on at 500 (left, right, forward in
# word 0). This switches to the fourth,fifth or sixth state (in state1)
# which persists until the end.


#---STEP 1
from parseBSClass import parser_BrainScaleS

import os
import sys
import numpy as np

from pysthal.command_line_util import init_logger
import pylogging

#---STEP 2 main--
#select
#sent = 0 #turn left
sent = 1 #turn right
#sent = 2 #move forward

parserObj = parser_BrainScaleS()

logger = pylogging.get("guidebook")

init_logger("WARN", [
    ("guidebook", "DEBUG"),
    ("marocco", "DEBUG"),
    #("Calibtic", "DEBUG"),
    #("sthal", "INFO")
])

parserObj.FSABSS.brainScalesHeader()
duration = 1000.0

#####SETUP NETWORK


StartState = 0
TurnState = 1
MoveState = 2
TurnLeftFinalState = 3
TurnRightFinalState = 4
MoveForwardFinalState = 5
TurnWord = 0
MoveWord = 1
LeftWord = 2
RightWord = 3
ForwardWord = 4

#Allocate Neurons 
numStates = 6
numWords = 5
statePops = parserObj.makePopulations(numStates)
state0Pop = statePops[0]
state1Pop = statePops[1]
wordPops = parserObj.makePopulations(numWords)
word0Pop = wordPops[0]
word1Pop = wordPops[1]

state0Pop.record()
state1Pop.record()
word0Pop.record()
word1Pop.record()

parserObj.makeCAs(statePops,numStates)
parserObj.makeCAs(wordPops,numWords)

excSpikeTimes0 = [100]
spikeSources0 = parserObj.FSABSS.getSpikeSources(excSpikeTimes0)

excSpikeTimes1 = [300]
spikeSources1 = parserObj.FSABSS.getSpikeSources(excSpikeTimes1)

excSpikeTimes2 = [500]
spikeSources2 = parserObj.FSABSS.getSpikeSources(excSpikeTimes2)

sourceProjections = []
sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources0,state0Pop,StartState)]

if (sent == 0):
    sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources1,word0Pop,TurnWord)] 
    sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources2,word0Pop,LeftWord)] #careful with words > 2
elif (sent == 1):
    sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources1,word0Pop,TurnWord)] 
    sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources2,word1Pop,RightWord-3)] #careful with words > 2

else:
    sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources1,word0Pop,MoveWord)] 
    sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources2,word1Pop,ForwardWord-3)] #careful with words > 2


#define the transition table
parserObj.addTransition(StartState,TurnState,TurnWord,statePops,wordPops)
parserObj.addTransition(StartState,MoveState,MoveWord,statePops,wordPops)
parserObj.addTransition(TurnState,TurnRightFinalState,RightWord,statePops,wordPops)
parserObj.addTransition(TurnState,TurnLeftFinalState,LeftWord,statePops,wordPops)
parserObj.addTransition(MoveState,MoveForwardFinalState,ForwardWord,statePops,wordPops)

parserObj.FSABSS.brainScalesMap(state0Pop,[198])
parserObj.FSABSS.brainScalesMap(state1Pop,[197])
parserObj.FSABSS.brainScalesMap(word0Pop,[196])
parserObj.FSABSS.brainScalesMap(word1Pop,[199])

parserObj.FSABSS.runSim(10) # run pynn after mapping

#Set the hardware weights and run properly.
parserObj.setHardwareWeights(sourceProjections)

#parserObj.FSABSS.setHardwareWeights(parserObj.CAProjections,
#                                    sourceProjections,
#                                    parserObj.halfProjections,[],
#                                    parserObj.stopProjections,[],[])

parserObj.FSABSS.brainScalesFooter()
parserObj.FSABSS.runSim(duration)
np.savetxt("output/move0StateSpikes.txt", state0Pop.getSpikes())
np.savetxt("output/move1StateSpikes.txt", state1Pop.getSpikes())
np.savetxt("output/moveWord0Spikes.txt", word0Pop.getSpikes())
np.savetxt("output/moveWord1Spikes.txt", word1Pop.getSpikes())
parserObj.FSABSS.resetSim()

