# Turn left, turn right, or move forward
# This runs through all three with a stop signal to kill the final state.

# The output are in ouput/move*.txt 


#---STEP 1
from parseBSClass import parser_BrainScaleS

import os
import sys
import numpy as np

from pysthal.command_line_util import init_logger
import pylogging

#---STEP 2 main--
parserObj = parser_BrainScaleS()

logger = pylogging.get("guidebook")

init_logger("WARN", [
    ("guidebook", "DEBUG"),
    ("marocco", "DEBUG"),
    #("Calibtic", "DEBUG"),
    #("sthal", "INFO")
])

parserObj.FSABSS.brainScalesHeader()
duration = 3000.0

#####SETUP NETWORK
StartState = 0
TurnState = 1
MoveState = 2
TurnLeftFinalState = 3
TurnRightFinalState = 4
MoveForwardFinalState = 5
TurnWord = 0
MoveWord = 1
LeftWord = 2
RightWord = 3
ForwardWord = 4

#Allocate Neurons 
numStates = 6
numWords = 5
statePops = parserObj.makePopulations(numStates)
state0Pop = statePops[0]
state1Pop = statePops[1]
wordPops = parserObj.makePopulations(numWords)
word0Pop = wordPops[0]
word1Pop = wordPops[1]

state0Pop.record()
state1Pop.record()
word0Pop.record()
word1Pop.record()

parserObj.makeCAs(statePops,numStates)
parserObj.makeCAs(wordPops,numWords)

excSpikeTimes0 = [100,1100,2100]
spikeSources0 = parserObj.FSABSS.getSpikeSources(excSpikeTimes0)

excSpikeTimes1 = [300,1300] #turn
spikeSources1 = parserObj.FSABSS.getSpikeSources(excSpikeTimes1)

excSpikeTimes2 = [500] #left
spikeSources2 = parserObj.FSABSS.getSpikeSources(excSpikeTimes2)

excSpikeTimes3 = [1500] #right 
spikeSources3 = parserObj.FSABSS.getSpikeSources(excSpikeTimes3)

excSpikeTimes4 = [2300] #move
spikeSources4 = parserObj.FSABSS.getSpikeSources(excSpikeTimes4)

excSpikeTimes5 = [2500] #forward
spikeSources5 = parserObj.FSABSS.getSpikeSources(excSpikeTimes5)

inhSpikeTimes0 = [900,910,1900,1910,2900,2910]
inhSpikeSources = parserObj.FSABSS.getSpikeSources(inhSpikeTimes0)

sourceProjections = []
sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources0,state0Pop,StartState)]
sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources1,word0Pop,TurnWord)] #turn
sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources2,word0Pop,2)] #left
sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources3,word1Pop,0)] #right
sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources4,word0Pop,1)] #move
sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources5,word1Pop,1)] #forward

sourceStopProjections = []
sourceStopProjections = sourceStopProjections+ [parserObj.FSABSS.turnOffStateFromSpikeSource(inhSpikeSources,state1Pop,0)]
sourceStopProjections = sourceStopProjections+ [parserObj.FSABSS.turnOffStateFromSpikeSource(inhSpikeSources,state1Pop,1)]
sourceStopProjections = sourceStopProjections+ [parserObj.FSABSS.turnOffStateFromSpikeSource(inhSpikeSources,state1Pop,2)]

#define the transition table
parserObj.addTransition(StartState,TurnState,TurnWord,statePops,wordPops)
parserObj.addTransition(StartState,MoveState,MoveWord,statePops,wordPops)
parserObj.addTransition(TurnState,TurnRightFinalState,RightWord,statePops,wordPops)
parserObj.addTransition(TurnState,TurnLeftFinalState,LeftWord,statePops,wordPops)
parserObj.addTransition(MoveState,MoveForwardFinalState,ForwardWord,statePops,wordPops)

parserObj.FSABSS.brainScalesMap(state0Pop,[197])
parserObj.FSABSS.brainScalesMap(state1Pop,[198])
parserObj.FSABSS.brainScalesMap(word0Pop,[196])
parserObj.FSABSS.brainScalesMap(word1Pop,[199])

parserObj.FSABSS.runSim(10) # run pynn after mapping

#Set the hardware weights and run properly.
#parserObj.setHardwareWeights(sourceProjections)

parserObj.FSABSS.setHardwareWeights(parserObj.CAProjections,
                                    sourceProjections,
                                    parserObj.halfProjections,[],
                                    parserObj.stopProjections,
                                    sourceStopProjections,[])

parserObj.FSABSS.brainScalesFooter()
parserObj.FSABSS.runSim(duration)
np.savetxt("output/stop0StateSpikes.txt", state0Pop.getSpikes())
np.savetxt("output/stop1StateSpikes.txt", state1Pop.getSpikes())
np.savetxt("output/stopWord0Spikes.txt", word0Pop.getSpikes())
np.savetxt("output/stopWord1Spikes.txt", word1Pop.getSpikes())
parserObj.FSABSS.resetSim()

