# Turn left, or turn right 
# The first spike source starts the parse, the second switches to the turn 
# state.  The third switches to either turn left or turn right.  Change
# the comment for that (look for select).

#---STEP 1
from parseBSClass import parser_BrainScaleS

import os
import sys
import numpy as np

from pysthal.command_line_util import init_logger
import pylogging

#---STEP 2 main--
parserObj = parser_BrainScaleS()

logger = pylogging.get("guidebook")

init_logger("WARN", [
    ("guidebook", "DEBUG"),
    ("marocco", "DEBUG"),
    #("Calibtic", "DEBUG"),
    #("sthal", "INFO")
])

parserObj.FSABSS.brainScalesHeader()
duration = 1000.0

#####SETUP NETWORK
StartState = 0
TurnState = 1
TurnRightFinalState = 2
TurnLeftFinalState = 3
TurnWord = 0
RightWord = 1
LeftWord = 2

#Allocate Neurons 
numStates = 4
numWords = 3
statePops = parserObj.makePopulations(numStates)
state0Pop = statePops[0]
state1Pop = statePops[1]
wordPops = parserObj.makePopulations(numWords)
wordPop = wordPops[0]

state0Pop.record()
state1Pop.record()
wordPop.record()

parserObj.makeCAs(statePops,numStates)
parserObj.makeCAs(wordPops,numWords)

excSpikeTimes0 = [100]
spikeSources0 = parserObj.FSABSS.getSpikeSources(excSpikeTimes0)

excSpikeTimes1 = [300]
spikeSources1 = parserObj.FSABSS.getSpikeSources(excSpikeTimes1)

excSpikeTimes2 = [500]
spikeSources2 = parserObj.FSABSS.getSpikeSources(excSpikeTimes2)

sourceProjections = []
sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources0,state0Pop,StartState)]

sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources1,wordPop,TurnWord)] 

#Select word comment out RightWord or leftWord undone
sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources2,wordPop,RightWord)] #careful with words > 2
#sourceProjections = sourceProjections + [parserObj.FSABSS.turnOnStateFromSpikeSource(spikeSources2,wordPop,LeftWord)] #careful with words > 2

#define the transition table 
parserObj.addTransition(StartState,TurnState,TurnWord,statePops,wordPops)
parserObj.addTransition(TurnState,TurnRightFinalState,RightWord,statePops,wordPops)
parserObj.addTransition(TurnState,TurnLeftFinalState,LeftWord,statePops,wordPops)


parserObj.FSABSS.brainScalesMap(state0Pop,[297])
parserObj.FSABSS.brainScalesMap(state1Pop,[298])
parserObj.FSABSS.brainScalesMap(wordPop,[296])
parserObj.FSABSS.runSim(10) # run pynn after mapping

#Set the hardware weights and run properly.
parserObj.setHardwareWeights(sourceProjections)

#parserObj.FSABSS.setHardwareWeights(parserObj.CAProjections,
#                                    sourceProjections,
#                                    parserObj.halfProjections,[],
#                                    parserObj.stopProjections,[],[])

parserObj.FSABSS.brainScalesFooter()
parserObj.FSABSS.runSim(duration)
np.savetxt("output/tLR0StateSpikes.txt", state0Pop.getSpikes())
np.savetxt("output/tLR1StateSpikes.txt", state1Pop.getSpikes())
np.savetxt("output/tLRWordSpikes.txt", wordPop.getSpikes())
parserObj.FSABSS.resetSim()

